// SimpleHistoryExample.cpp : Defines the entry point for the console application.
//
#include <blpapi_session.h>
#include <blpapi_eventdispatcher.h>

#include <blpapi_event.h>
#include <blpapi_message.h>
#include <blpapi_element.h>
#include <blpapi_name.h>
#include <blpapi_request.h>
#include <blpapi_subscriptionlist.h>
#include <blpapi_defs.h>
#include <blpapi_exception.h>

#include <iostream>
#include <sstream>
#include <vector>
#include <string>
#include <stdlib.h>
#include <string.h>

#include "mathlink.h"

using namespace std;
using namespace BloombergLP;
using namespace blpapi;

namespace {
    const Name SECURITY_DATA("securityData");
    const Name FIELD_DATA("fieldData");
};

class SimpleHistoryExample
{
	std::string         d_host;
    int                 d_port;
	
	//Bloomberg Original Function
    void printUsage()
    {
		std::cout << "Usage:" << std::endl
			<< "	Retrieve reference data from ServerApi" << std::endl
			<< "		[-ip 		<ipAddress	= localhost>" << std::endl
			<< "		[-p 		<tcpPort	= 8194>" << std::endl;
    }

	

    public:
	//Modified Bloomberg Function
    string Retrieve(const char* Ticker,const char* Field, const char* startDate, const char* endDate, const char* periodicitySelection = "DAILY", const char* periodicityAdjustment = "Actual")
    {
        string output = "Failed";
		d_host = "localhost";
        d_port = 8194;

        SessionOptions sessionOptions;
        sessionOptions.setServerHost(d_host.c_str());
        sessionOptions.setServerPort(d_port);

        std::cout << "Connecting to " <<  d_host << ":" << d_port << std::endl;
        Session session(sessionOptions);
        if (!session.start()) {
            std::cerr <<"Failed to start session." << std::endl;
			return "Failed to start session";
        }
        if (!session.openService("//blp/refdata")) {
            std::cerr <<"Failed to open //blp/refdata" << std::endl;
            return "Failed to open //blp/refdata";
        }
        Service refDataService = session.getService("//blp/refdata");
        Request request = refDataService.createRequest("HistoricalDataRequest");
        request.getElement("securities").appendValue(Ticker);
        request.getElement("fields").appendValue(Field);
        request.set("periodicityAdjustment", periodicityAdjustment);
        request.set("periodicitySelection", periodicitySelection);
        request.set("startDate", startDate);
        request.set("endDate", endDate);
        //request.set("maxDataPoints", 100);

        std::cout << "Sending Request: " << request << std:: endl;
        session.sendRequest(request);

        while (true) {
            Event event = session.nextEvent();
            MessageIterator msgIter(event);
            while (msgIter.next()) {
                Message msg = msgIter.message();
                Element securities = msg.asElement();
				if (securities.numElements()>0){
					Element securityData = msg.getElement(SECURITY_DATA); //get security
					Element fieldDataArray = securityData.getElement(FIELD_DATA);
					output = "{";
					for(size_t j = 0; j <fieldDataArray.numValues();++j){
						Element fieldData = fieldDataArray.getValueAsElement(j); //get each element in the security, every single data point (date, data) is an element
						for(size_t k=0;k < fieldData.numElements();k=k+2){
							output += "{\"";
							Element field1 = fieldData.getElement(k); //date is an element
							Element field2 = fieldData.getElement(k+1); //data is an element
							output += field1.getValueAsString();
							output += "\", ";
							output += field2.getValueAsString();
							output += "}";
						}
						if(j < fieldDataArray.numValues()-1)
							output += ",";
					}
					output += "}";
						std::cout<<output<<std::endl;
				}
			}
            if (event.eventType() == Event::RESPONSE) {
                break;
			}
		}
		return output;
	}
};

void BBRetrieve(const char* Ticker,const char* Field, const char* startDate, const char* endDate, const char* periodicitySelection = "DAILY", const char* periodicityAdjustment = "Actual")
{
	string temp;
	//Create SimpleHistoryExample Class Object
	SimpleHistoryExample Example;
	//Retrieve string created by the example object
	temp = Example.Retrieve(Ticker,Field, startDate, endDate, periodicitySelection, periodicityAdjustment);
	std::cout<<"Temp: "<<temp<<std::endl;
	//Put string to Mathematica
	MLPutString(stdlink, temp.c_str());
	return;
}

#if MACINTOSH_MATHLINK

int main( int argc, char* argv[])
{
	/* Due to a bug in some standard C libraries that have shipped with
	 * MPW, zero is passed to MLMain below.  (If you build this program
	 * as an MPW tool, you can change the zero to argc.)
	 */
	argc = argc; /* suppress warning */
	return MLMain( 0, argv);
}

#elif WINDOWS_MATHLINK

#if __BORLANDC__
#pragma argsused
#endif

int PASCAL WinMain( HINSTANCE hinstCurrent, HINSTANCE hinstPrevious, LPSTR lpszCmdLine, int nCmdShow)
{
	char  buff[512];
	char FAR * buff_start = buff;
	char FAR * argv[32];
	char FAR * FAR * argv_end = argv + 32;

	hinstPrevious = hinstPrevious; /* suppress warning */

	if( !MLInitializeIcon( hinstCurrent, nCmdShow)) return 1;
	MLScanString( argv, &argv_end, &lpszCmdLine, &buff_start);
	return MLMain( (int)(argv_end - argv), argv);
}

#else

int main(int argc, char* argv[])
{
	return MLMain(argc, argv);
}

#endif
