// SimpleRefDataExample.cpp : Defines the entry point for the console application.
//
#include <blpapi_session.h>
#include <blpapi_eventdispatcher.h>

#include <blpapi_event.h>
#include <blpapi_message.h>
#include <blpapi_element.h>
#include <blpapi_name.h>
#include <blpapi_request.h>
#include <blpapi_subscriptionlist.h>
#include <blpapi_defs.h>
#include <blpapi_exception.h>

#include <iostream>
#include <sstream>
#include <fstream>
#include <vector>
#include <string>
#include <stdlib.h>
#include <string.h>

#include "mathlink.h"

using namespace std;
using namespace BloombergLP;
using namespace blpapi;

namespace {
    const Name SECURITY_DATA("securityData");
    const Name SECURITY("security");
    const Name FIELD_DATA("fieldData");
	const Name SEQ_NUMBER("sequenceNumber");
    const Name RESPONSE_ERROR("responseError");
    const Name SECURITY_ERROR("securityError");
    const Name FIELD_EXCEPTIONS("fieldExceptions");
    const Name FIELD_ID("fieldId");
    const Name ERROR_INFO("errorInfo");
    const Name CATEGORY("category");
    const Name MESSAGE("message");
    const Name SESSION_TERMINATED("SessionTerminated");
    const Name SESSION_STARTUP_FAILURE("SessionStartupFailure");
};

class SimpleRefDataExample
{
	std::string         d_host;
    int                 d_port;

    void printUsage()
    {
		std::cout << "Usage:" << std::endl
			<< "	Retrieve reference data " << std::endl
			<< "		[-ip 		<ipAddress	= localhost>" << std::endl
			<< "		[-p 		<tcpPort	= 8194>" << std::endl;
		return;
    }

	void printErrorInfo(const char *leadingStr, const Element &errorInfo)
    {
        std::cout << leadingStr
            << errorInfo.getElementAsString(CATEGORY)
            << " ("
            << errorInfo.getElementAsString(MESSAGE)
            << ")" << std::endl;
    }

	string processEvent(Event &event)
    {
        string output = "Failed";
		MessageIterator msgIter(event);
        while (msgIter.next()) {
            Message msg = msgIter.message();
            if (msg.hasElement(RESPONSE_ERROR)) {
                printErrorInfo("REQUEST FAILED: ", 
                    msg.getElement(RESPONSE_ERROR));
                continue;
            }
            output = processMessage(msg);
        }
		return output;
    }

	string processMessage(Message &msg)
    {
		string s;
		stringstream out;
	
		string output;
		Element securities = msg.getElement(SECURITY_DATA);
		size_t numSecurities = securities.numValues();
		out<<numSecurities;
		std::cout << "Processing " << (unsigned int)numSecurities << " securities:"<< std::endl;
		output= "";
		for (size_t i = 0; i < numSecurities; ++i) {
			Element security = securities.getValueAsElement(i);
			std::string ticker = security.getElementAsString(SECURITY);
			std::cout << "\nTicker: " + ticker << std::endl;
			output+=ticker+", ";
			if (security.hasElement("securityError")) {
				printErrorInfo("\tSECURITY FAILED: ",
				security.getElement(SECURITY_ERROR));
				continue;
			}
			if (security.hasElement(FIELD_DATA)) {
				const Element fields = security.getElement(FIELD_DATA);
				if (fields.numElements() > 0) {	
					std::cout << "FIELD\t\tVALUE"<<std::endl;
					std::cout << "-----\t\t-----"<< std::endl;
					size_t numElements = fields.numElements();
					for (size_t j = 0; j < numElements; ++j) {
						Element field = fields.getElement(j);
						std::cout << field.name() << "\t\t" <<
						field.getValueAsString() << std::endl;
						output +=field.getValueAsString();
						if(j < fields.numValues()-1)
							output += ",";
					}
				}
			}
			if(i<numSecurities-1)
				output+=", ";
		}
		output+="";
		return output;
    }

    public:
	string Retrieve(const char* Ticker, const char* Field)
    {
        string output = "Failed";
		string outputtemp;
		d_host = "localhost";
        d_port = 8194;
		
        SessionOptions sessionOptions;
        sessionOptions.setServerHost(d_host.c_str());
        sessionOptions.setServerPort(d_port);
		char buff[100000];
		char *token;
		char seps[] = "&";


        std::cout << "Connecting to " <<  d_host << ":" 
                  << d_port << std::endl;
        Session session(sessionOptions);
        if (!session.start()) {
            std::cerr <<"Failed to start session." << std::endl;
            return "Failed to open";
        }
        if (!session.openService("//blp/refdata")) {
            std::cerr <<"Failed to open //blp/refdata" << std::endl;
            return "Failed to open";
        }

        Service refDataService = session.getService("//blp/refdata");
        Request request = refDataService.createRequest("ReferenceDataRequest");

		// Add securities to request
        Element securities = request.getElement("securities");
		//securities.appendValue(Ticker);
		
		strncpy(buff,Ticker,100000);
		token = strtok(buff,seps);
		while (token!=NULL)
		{
			request.append("securities", token);
			std::cout<<token<<endl;
			token = strtok(NULL,seps);
		}
		
		// Add fields to request
        Element fields = request.getElement("fields");
        fields.appendValue(Field);

        std::cout << "Sending Request: " << request << std::endl;
        session.sendRequest(request);

		bool done = false;
		output = "{";
        while (!done) {
            Event event = session.nextEvent();
            if (event.eventType() == Event::PARTIAL_RESPONSE) {
                std::cout << "Processing Partial Response" << std::endl;
                outputtemp = processEvent(event);
				if (output[output.length()-1]=='{')
					output = output+outputtemp;
				else
					output = output+", "+outputtemp;
			}
            else if (event.eventType() == Event::RESPONSE) {
                std::cout << "Processing Response" << std::endl;
                outputtemp = processEvent(event);
				done = true;
				if(output[output.length()-1]=='{')
					output = outputtemp+"}";
				else
					output = output+", "+outputtemp+"}";
            } else {
                MessageIterator msgIter(event);
                while (msgIter.next()) {
                    Message msg = msgIter.message();
                    if (event.eventType() == Event::SESSION_STATUS) {
                        if (msg.messageType() == SESSION_TERMINATED ||
                            msg.messageType() == SESSION_STARTUP_FAILURE) {
                            done = true;
							output = "Failed";
                        }
                    }
                }
            }
        }
		return output;
	}
};

void BBRetrieveCurrent(const char* Ticker, const char* Field)
{
	string temp;
	//Create SimpleHistoryExample Class Object
	SimpleRefDataExample Example;
	//Retrieve string created by the example object
	temp = Example.Retrieve(Ticker,Field);
	std::cout<<"Temp: "<<temp<<std::endl;
	//Put string to Mathematica
	MLPutString(stdlink, temp.c_str());
	return;
}

#if MACINTOSH_MATHLINK
int main( int argc, char* argv[])
{
	/* Due to a bug in some standard C libraries that have shipped with
	 * MPW, zero is passed to MLMain below.  (If you build this program
	 * as an MPW tool, you can change the zero to argc.)
	 */
	argc = argc; /* suppress warning */
	return MLMain( 0, argv);
}

#elif WINDOWS_MATHLINK

#if __BORLANDC__
#pragma argsused
#endif

int PASCAL WinMain( HINSTANCE hinstCurrent, HINSTANCE hinstPrevious, LPSTR lpszCmdLine, int nCmdShow)
{
	char  buff[512];
	char FAR * buff_start = buff;
	char FAR * argv[32];
	char FAR * FAR * argv_end = argv + 32;

	hinstPrevious = hinstPrevious; /* suppress warning */

	if( !MLInitializeIcon( hinstCurrent, nCmdShow)) return 1;
	MLScanString( argv, &argv_end, &lpszCmdLine, &buff_start);
	return MLMain( (int)(argv_end - argv), argv);
}

#else

int main(int argc, char* argv[])
{
	return MLMain(argc, argv);
}

#endif


